import click
from .client import Hbtn
from .helpers import Settings, create_files

settings = Settings()


@click.group()
def main():
    """Create all necessary files of a project. by supplying one or more URL.
    """
    pass


@main.command()
@click.argument('urls', required=True, nargs=-1)
def fetch(urls):
    """Fetch project files and creates them.\n
    Example:
        hbtn fetch https://intranet.hbtn.io/projects/<project_id>
        hbtn fetch <URL1> <URL2>
    """
    if not settings.file_status:
        exit("Settings.ini not found: run hbtn setup :)")
    location, username, password = settings.read_from_file
    client = Hbtn(username, password)
    for url in urls:
        create_files(location, client.fetch_project_details(url))


@main.command()
def setup():
    """
    Create settings.ini.
    Command: hbtn setup
    """
    settings.setup()


if __name__ == "__main__":
    main()



import os
import pathlib
from base64 import b64encode, b64decode
from configparser import ConfigParser
from typing import List, Tuple, Dict, Any, Union


def encrypted(data: str) -> str:
    """
    Returns a Decoded base64 string.
    """
    return b64encode(data.encode('ascii')).decode('ascii')


def decrypted(data: str) -> str:
    """
    Returns an Encoded password with base64.
    """

    return b64decode(data.encode('ascii')).decode('ascii')


def create_files(root_dir: str, project_details: Union[Dict[str, List[Dict[str, list]]], dict]):
    """
    Create file in a root_dir.
    """
    github_repo = project_details['github_repo']
    github_dir = project_details['github_dir']

    for task in project_details["tasks"]:
        for file in task['github_file']:
            formatted_file = f"{root_dir}/{github_repo}/{github_dir}/{file}"
            dirs = '/'.join(formatted_file.split('/')[:-1]) + '/' \
                if not formatted_file.endswith('/') else formatted_file
            pathlib.Path(dirs).mkdir(parents=True, exist_ok=True)

            with open(formatted_file, 'w') as f:
                f.write("")


class Settings:
    __SETTINGS_FILE = "settings.ini"

    def __init__(self):
        self.__config = ConfigParser()
        self.file_status = os.path.exists(self.__SETTINGS_FILE)

    @property
    def read_from_file(self) -> Tuple:
        """
        Read from settings.ini
        :returns: Tuple(location, username, password)
        """
        assert self.file_status, f"{self.__SETTINGS_FILE} not found"
        self.__config.read(self.__SETTINGS_FILE)
        # Import settings variables
        return (
            self.__config['settings']['location'],
            self.__config['settings']['username'],
            decrypted(self.__config['settings']['password'])
        )

    def write_to_file(self, **kwargs: Dict[str, Any]):
        """
        Write to settings.ini.
        """

        valid_keys = ['username', 'password', 'location']
        self.__config["settings"] = {
            k: v for k, v in kwargs.items() if k in valid_keys
        }
        with open(self.__SETTINGS_FILE, 'w') as conf:
            self.__config.write(conf)

    def setup(self):
        """
        Create a settings.ini file with user credentials.
        """
        messages = [
            "Please enter you Holberton email: ",
            "Please enter your Holberton password (don't worry passwd will be encrypted): ",
            "Please enter full path where you want to save future projects: "
        ]
        settings_ini_variables = ["username", 'password', 'location']

        settings_ini = {}
        for msg, var in zip(messages, settings_ini_variables):
            user_input = str(input(msg))

            if var == "location":
                while not os.path.exists(user_input):
                    print("[!]: SUPPLIED PATH DOES NOT EXIST.")
                    user_input = str(input(msg))
            settings_ini[var] = encrypted(user_input) if var == "password" else user_input

        self.write_to_file(**settings_ini)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='hbtn-cli',
    version='2.0.0',
    author='Malek Salem',
    author_email='1419@holbertonschool.com',
    license="MIT License",
    description='Automates Holberton School projects creation.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/maleksal/hbtn-cli',
    packages=find_packages(),
    install_requires=[
        "beautifulsoup4==4.10.0",
        "Click==7.1",
        "requests",
        "lxml",
        "wheel",
    ],
    python_requires='>=3.5',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: Unix"
    ],
    entry_points='''
        [console_scripts]
        hbtn=core.interface:main
    '''
)



"""
Hbtn Module
"""
from typing import List, Dict, Union, Any

import requests
from bs4 import BeautifulSoup

JsonType = Dict[str, Union[List[Dict[str, Union[list, Any]]], Any]]


class Hbtn:
    """
    Class that authenticates to the intranet website,
    and fetches json data of a project referenced by URL.
    """

    __loginURL = "https://intranet.hbtn.io/auth/sign_in"
    __headers = {'user-agent': "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (HTML, like Gecko) "
                               "Chrome/80.0.3987.87 Safari/537.36"}

    def __init__(self, username: str, password: str):
        """
        :param username:    intranet username.
        :param password:    intranet password.
        """
        # Initiate session
        with requests.Session() as session:
            session.headers.update(self.__headers)
            self.__session = session
        assert self.authenticate(
            username,
            password), "Double check your credentials [Authentication Failed]"

    @staticmethod
    def get_token(html_content: str) -> str:
        """
        Returns extracted auth token from html
        """
        soup = BeautifulSoup(html_content, features='lxml')
        return soup.find('input', attrs={'name': 'authenticity_token'})['value']

    @staticmethod
    def preprocess_data(json_data: JsonType) -> JsonType:
        """
        Cleans the retrieved data.
        """
        return {
            "name": json_data["name"],
            'github_repo': json_data['tasks'][0]['github_repo'],
            'github_dir': json_data['tasks'][0]['github_dir'],
            "tasks": [
                {
                    'title': task['title'],
                    'github_file': [
                        file.strip() for file in task['github_file'].split(',')
                        if file.split('.')[-1] not in ['png', 'jpeg', 'icon', 'jpg']
                    ]
                }
                for task in json_data['tasks']]
        }

    def get_login_page(self) -> str:
        """Get login page.
        :return: response.text
        """
        with self.__session.get(self.__loginURL) as res:
            return res.text

    def authenticate(self, username: str, password: str) -> bool:
        """
        Handles authentication with website using username && password.
        """
        login_page = self.get_login_page()
        payload = {
            'authenticity_token': self.get_token(login_page),
            'user[login]': username,
            'user[password]': password,
            'user[remember_me]': '0',
            'commit': 'Log ' + 'in'}

        # login to website
        with self.__session.post(self.__loginURL, data=payload) as res:
            return 'Invalid Email or password.' not in res.text

    def fetch_project_details(self, url: str) -> Union[JsonType, Dict]:
        """Fetch project details referenced by project URL.
        :param url: project URL.
        :return:    json data or empty dict.
        """
        with self.__session.get(url + ".json") as res:
            if res.status_code in range(200, 299):
                data = self.preprocess_data(res.json())
                data['tasks'].append(
                    {  # Add README.md file :)
                        'title': "README.md file",
                        'github_file': ["README.md"]
                    }
                )
                return data
            return {}
